@@ -32,10 +32,11 @@
  # ###################################################################
  ##
 
-r"""
+r"""Solve a one-dimensional diffusion equation under different conditions.
+
 To run this example from the base :term:`FiPy` directory, type::
     
-    $ examples/diffusion/mesh1D.py
+    $ python examples/diffusion/mesh1D.py
     
 at the command line. Different stages of the example should be displayed,
 along with prompting messages in the terminal.
@@ -50,14 +51,14 @@ diffusivity and fixed value boundary conditions such that,
    \frac{\partial \phi}{\partial t} = D \nabla^2 \phi.
 
 The first step is to define a one dimensional domain with 50 solution
-points. The :class:`~fipy.meshes.numMesh.grid1D.Grid1D` object represents a linear structured grid. The
+points. The :class:`~fipy.meshes.grid1D.Grid1D` object represents a linear structured grid. The
 parameter ``dx`` refers to the grid spacing (set to unity here).
 
 >>> from fipy import *
 
 >>> nx = 50
 >>> dx = 1.
->>> mesh = Grid1D(nx = nx, dx = dx)
+>>> mesh = Grid1D(nx=nx, dx=dx)
 
 :term:`FiPy` solves all equations at the centers of the cells of the mesh. We
 thus need a :class:`~fipy.variables.cellVariable.CellVariable` object to hold the values of the
@@ -74,7 +75,7 @@ We'll let
 for now. 
 
 The set of boundary conditions are given to the equation as a Python
-:keyword:`tuple` or :keyword:`list` (the distinction is not generally important to :term:`FiPy`).
+tuple or list (the distinction is not generally important to :term:`FiPy`).
 The boundary conditions 
 
 .. math::
@@ -97,17 +98,19 @@ and a set of faces over which they apply.
    Only faces around the exterior of the mesh can be used for boundary
    conditions.
 
-For example, here the exterior faces on the left of the domain are
-extracted by ``mesh``:meth:`~fipy.meshes.common.mesh.Mesh.getFacesLeft``. A :class:`~fipy.boundaryConditions.fixedValue.FixedValue` boundary condition is
-created with these faces and a value (``valueLeft``). 
+For example, here the exterior faces on the left of the domain are extracted by
+``mesh``.\ :attr:`~fipy.meshes.abstractMesh.AbstractMesh.facesLeft`. The boundary
+conditions is applied using
+``phi``\. :meth:`~fipy.variables.variable.Variable.constrain` with tthese faces and
+a value (``valueLeft``).
 
->>> BCs = (FixedValue(faces=mesh.getFacesRight(), value=valueRight),
-...        FixedValue(faces=mesh.getFacesLeft(), value=valueLeft))
+>>> phi.constrain(valueRight, mesh.facesRight)
+>>> phi.constrain(valueLeft, mesh.facesLeft)
 
 .. note::
     
    If no boundary conditions are specified on exterior faces, the default
-   boundary condition is ``FixedFlux(faces=someFaces, value=0.)``, equivalent to
+   boundary condition is equivalent to a zero gradient, equivalent to
    :math:`\vec{n} \cdot \nabla \phi \rvert_\text{someFaces} = 0`.
 
 If you have ever tried to numerically solve
@@ -140,7 +143,7 @@ this example is being run automatically as a test. We accomplish this by
 having Python check if this script is the "``__main__``" script, which will
 only be true if we explicitly launched it and not if it has been imported
 by another script such as the automatic tester. The factory function
-:func:`Viewer` returns a suitable viewer depending on available
+:func:`~fipy.viewers.Viewer` returns a suitable viewer depending on available
 viewers and the dimension of the mesh.
 
 .. index::
@@ -159,12 +162,12 @@ diffusion problem is given by
 :math:`\phi = 1 - \erf(x/2\sqrt{D t})`. If the :term:`SciPy` library is available,
 the result is tested against the expected profile: 
 
->>> x = mesh.getCellCenters()[0]
+>>> x = mesh.cellCenters[0]
 >>> t = timeStepDuration * steps
 
 >>> try:
-...     from scipy.special import erf
-...     phiAnalytical.setValue(1 - erf(x / (2 * sqrt(D * t))))
+...     from scipy.special import erf # doctest: +SCIPY
+...     phiAnalytical.setValue(1 - erf(x / (2 * numerix.sqrt(D * t)))) # doctest: +SCIPY
 ... except ImportError:
 ...     print "The SciPy library is not available to test the solution to \
 ... the transient diffusion equation"
@@ -173,12 +176,11 @@ We then solve the equation by repeatedly looping in time:
 
 >>> for step in range(steps):
 ...     eqX.solve(var=phi,
-...               boundaryConditions=BCs,
 ...               dt=timeStepDuration)
 ...     if __name__ == '__main__':
 ...         viewer.plot()
 
->>> print phi.allclose(phiAnalytical, atol = 7e-4)
+>>> print phi.allclose(phiAnalytical, atol = 7e-4) # doctest: +SCIPY
 1
 
 >>> if __name__ == '__main__':
@@ -187,6 +189,7 @@ We then solve the equation by repeatedly looping in time:
 .. image:: mesh1Dexplicit.*
    :width: 90%
    :align: center
+   :alt: solution to diffusion problem evolved by explicit time steps
 
 -----
 
@@ -214,15 +217,14 @@ reset the problem
 and rerun with much larger time steps
 
 >>> timeStepDuration *= 10
->>> steps /= 10
+>>> steps //= 10
 >>> for step in range(steps):
 ...     eqI.solve(var=phi,
-...               boundaryConditions=BCs,
 ...               dt=timeStepDuration)
 ...     if __name__ == '__main__':
 ...         viewer.plot()
 
->>> print phi.allclose(phiAnalytical, atol = 2e-2)
+>>> print phi.allclose(phiAnalytical, atol = 2e-2) # doctest: +SCIPY
 1
 
 >>> if __name__ == '__main__':
@@ -231,6 +233,7 @@ and rerun with much larger time steps
 .. image:: mesh1Dimplicit.*
    :width: 90%
    :align: center
+   :alt: solution to diffusion problem evolved by implicit time steps
 
 Note that although much larger *stable* timesteps can be taken with this
 implicit version (there is, in fact, no limit to how large an implicit
@@ -257,17 +260,15 @@ of the fully implicit scheme to drive down the error
 
 >>> for step in range(steps - 1):
 ...     eqCN.solve(var=phi,
-...                boundaryConditions=BCs,
 ...                dt=timeStepDuration)
 ...     if __name__ == '__main__':
 ...         viewer.plot()
 >>> eqI.solve(var=phi,
-...           boundaryConditions=BCs,
 ...           dt=timeStepDuration)
 >>> if __name__ == '__main__':
 ...     viewer.plot()
 
->>> print phi.allclose(phiAnalytical, atol = 3e-3)
+>>> print phi.allclose(phiAnalytical, atol = 3e-3) # doctest: +SCIPY
 1
 
 >>> if __name__ == '__main__':
@@ -287,9 +288,7 @@ equation
 
 is represented in :term:`FiPy` by
 
->>> DiffusionTerm(coeff=D).solve(var=phi,
-...                              boundaryConditions=BCs)
-...                                      
+>>> DiffusionTerm(coeff=D).solve(var=phi)
 
 >>> if __name__ == '__main__':
 ...     viewer.plot()
@@ -309,6 +308,7 @@ of :math:`10^{-10}`.
 .. image:: mesh1DsteadyState.*
    :width: 90%
    :align: center
+   :alt: steady-state solution to diffusion problem
        
 ------
 
@@ -330,8 +330,12 @@ we will need to declare time :math:`t` as a :class:`~fipy.variables.variable.Var
 
 and then declare our boundary condition as a function of this :class:`~fipy.variables.variable.Variable`
 
->>> BCs = (FixedValue(faces=mesh.getFacesLeft(), value=0.5 * (1 + sin(time))),
-...        FixedValue(faces=mesh.getFacesRight(), value=0.))
+>>> del phi.faceConstraints
+>>> valueLeft = 0.5 * (1 + numerix.sin(time))
+>>> phi.constrain(valueLeft, mesh.facesLeft)
+>>> phi.constrain(0., mesh.facesRight)
+
+>>> eqI = TransientTerm() == DiffusionTerm(coeff=D)
 
 When we update ``time`` at each timestep, the left-hand boundary
 condition will automatically update,
@@ -339,7 +343,7 @@ condition will automatically update,
 >>> dt = .1
 >>> while time() < 15:
 ...     time.setValue(time() + dt)
-...     eqI.solve(var=phi, dt=dt, boundaryConditions=BCs)
+...     eqI.solve(var=phi, dt=dt)
 ...     if __name__ == '__main__':
 ...         viewer.plot()
 
@@ -349,6 +353,7 @@ condition will automatically update,
 .. image:: mesh1DtimedBC.*
    :width: 90%
    :align: center
+   :alt: solution to diffusion problem with a time-dependent Dirichlet boundary condition
 
 ------
 
@@ -378,15 +383,15 @@ number of cells in the mesh :math:`N_i` satisfies :math:`N_i = 4 i + 2`,
 where :math:`i` is an integer. The mesh we've been using thus far is
 satisfactory, with :math:`N_i = 50` and :math:`i = 12`.
    
-Because :term:`FiPy` considers diffusion to be a flux from one :class:`~fipy.meshes.numMesh.cell.Cell` to the next,
-through the intervening :class:`~fipy.meshes.numMesh.face.Face`, we must define the non-uniform diffusion
+Because :term:`FiPy` considers diffusion to be a flux from one cell to the next,
+through the intervening face, we must define the non-uniform diffusion
 coefficient on the mesh faces
 
 .. index:: FaceVariable
 
 >>> D = FaceVariable(mesh=mesh, value=1.0)
->>> x = mesh.getFaceCenters()[0]
->>> D.setValue(0.1, where=(L / 4. <= x) & (x < 3. * L / 4.))
+>>> X = mesh.faceCenters[0]
+>>> D.setValue(0.1, where=(L / 4. <= X) & (X < 3. * L / 4.))
 
 The boundary conditions are a fixed value of 
 
@@ -398,10 +403,9 @@ to the left and a fixed flux of
     
 to the right:
 
-.. index:: FixedFlux
-
->>> BCs = (FixedValue(faces=mesh.getFacesLeft(), value=valueLeft),
-...        FixedFlux(faces=mesh.getFacesRight(), value=fluxRight))
+>>> phi = CellVariable(mesh=mesh)
+>>> phi.faceGrad.constrain([fluxRight], mesh.facesRight)
+>>> phi.constrain(valueLeft, mesh.facesLeft)
 
 We re-initialize the solution variable
     
@@ -409,8 +413,7 @@ We re-initialize the solution variable
     
 and obtain the steady-state solution with one implicit solution step
 
->>> DiffusionTerm(coeff = D).solve(var=phi, 
-...                                boundaryConditions = BCs)
+>>> DiffusionTerm(coeff = D).solve(var=phi)
 
 The analytical solution is simply
 
@@ -424,7 +427,7 @@ The analytical solution is simply
    
 or
 
->>> x = mesh.getCellCenters()[0]
+>>> x = mesh.cellCenters[0]
 >>> phiAnalytical.setValue(x)
 >>> phiAnalytical.setValue(10 * x - 9. * L / 4. , 
 ...                        where=(L / 4. <= x) & (x < 3. * L / 4.))
@@ -443,6 +446,7 @@ And finally, we can plot the result
 .. image:: mesh1Dnon-uniform.*
    :width: 90%
    :align: center
+   :alt: steady-state solution to diffusion problem with a non-uniform diffusivity
 
 ------
 
@@ -505,8 +509,8 @@ as
 We apply the same boundary conditions that we used for the uniform
 diffusivity cases
 
->>> BCs = (FixedValue(faces=mesh.getFacesRight(), value=valueRight),
-...        FixedValue(faces=mesh.getFacesLeft(), value=valueLeft))
+>>> phi[0].constrain(valueRight, mesh.facesRight)
+>>> phi[0].constrain(valueLeft, mesh.facesLeft)
 
 Although this problem does not have an exact transient solution, it
 can be solved in steady-state, with
@@ -515,8 +519,8 @@ can be solved in steady-state, with
 
    \phi(x) = 1 - \sqrt{\frac{x}{L}}
 
->>> x = mesh.getCellCenters()[0]
->>> phiAnalytical.setValue(1. - sqrt(x/L))
+>>> x = mesh.cellCenters[0]
+>>> phiAnalytical.setValue(1. - numerix.sqrt(x/L))
 
 We create a viewer to compare the different numbers of sweeps with the
 analytical solution from before.
@@ -529,10 +533,12 @@ analytical solution from before.
 As described above, an inner "sweep" loop is generally required for
 the solution of non-linear or multiple equation sets. Often a
 conditional is required to exit this "sweep" loop given some
-convergence criteria. Instead of using the :meth:`solve` method equation,
-when sweeping, it is often useful to call :meth:`sweep` instead. The
-:meth:`sweep` method behaves the same way as :meth:`solve`, but returns the
-residual that can then be used as part of the exit condition.
+convergence criteria. Instead of using the :meth:`~fipy.terms.term.Term.solve`
+method equation, when sweeping, it is often useful to call
+:meth:`~fipy.terms.term.Term.sweep` instead. The
+:meth:`~fipy.terms.term.Term.sweep` method behaves the same way as
+:meth:`~fipy.terms.term.Term.solve`, but returns the residual that can then be
+used as part of the exit condition.
 
 We now repeatedly run the problem with increasing numbers of
 sweeps.
@@ -546,7 +552,6 @@ sweeps.
 ...         # but "sweep" many times per time step
 ...         for sweep in range(sweeps):
 ...             res = eq.sweep(var=phi[0],
-...                            boundaryConditions=BCs,
 ...                            dt=timeStepDuration)
 ...         if __name__ == '__main__':
 ...             viewer.plot()
@@ -578,7 +583,6 @@ can just solve for it directly
 >>> res = 1e+10
 >>> while res > 1e-6:
 ...     res = eq.sweep(var=phi[0],
-...                    boundaryConditions=BCs,
 ...                    dt=timeStepDuration)
 
 
@@ -593,6 +597,7 @@ can just solve for it directly
 .. image:: mesh1Dvariable.*
    :width: 90%
    :align: center
+   :alt: solution to a diffusion problem a non-linear diffusivity
 
 ------
 
@@ -620,7 +625,6 @@ remaining lines, leaving::
      res = 1e+10
      while res > 1e-6:
          res = eq.sweep(var=phi[0],
-                        boundaryConditions=BCs,
                         dt=timeStepDuration)
 
      print phi[0].allclose(phiAnalytical, atol = 1e-1)
@@ -644,6 +648,9 @@ Most of the :term:`FiPy` examples will be a
 mixture of plain scripts and doctest documentation/tests.
 
 .. :term:`FiPy` replace:: `FiPy`
+
+.. .. bibmissing:: /documentation/refs.bib
+    :sort:
 """
 
 __docformat__ = 'restructuredtext'
