@@ -31,7 +31,8 @@
  # ###################################################################
  ##
 
-r"""
+r"""Model electrochemical superfill of copper with leveler and accelerator additives.
+
 This input file is a demonstration of the use of :term:`FiPy` for
 modeling copper superfill with leveler and accelerator
 additives. The material properties and experimental parameters
@@ -49,7 +50,7 @@ the ``numberOfSteps`` argument as follows,
 
 .. index:: runLeveler
 
->>> runLeveler(numberOfSteps=10, displayViewers=False, cellSize=0.25e-7)
+>>> runLeveler(numberOfSteps=10, displayViewers=False, cellSize=0.25e-7) # doctest: +GMSH
 1
 
 Change the ``displayViewers`` argument to ``True`` if you wish to see the
@@ -83,8 +84,8 @@ current density is governed by
    }{ R T }} - \exp{ \frac{ \left( 1 - \alpha_j \right) F \eta}{ R T
    }} \right) \right]
 
-where $j$ represents $S$ for suppressor, $A$
-for accelerator, $L$ for leveler and $V$ for vacant. This model
+where :math:`j` represents :math:`S` for suppressor, :math:`A`
+for accelerator, :math:`L` for leveler and :math:`V` for vacant. This model
 assumes a linear interpolation between the three cases of complete
 coverage for each additive or vacant substrate. The governing
 equations for the surfactants are given by, 
@@ -101,18 +102,17 @@ equations for the surfactants are given by,
    \\
    \theta_S &= 1 - \theta_A - \theta_L 
    \\
-   \theta_V = 0.
+   \theta_V &= 0.
      
-It has been found experimentally that $i_L = i_S$.
+It has been found experimentally that :math:`i_L = i_S`.
 
-If the surface reaches full coverage, the equations do not
-naturally prevent the coverage rising above full coverage due to
-the curvature terms. Thus, when $\theta_L + \theta_A = 1$ then the
-equation for accelerator becomes $\dot{ \theta_A } = -\dot{
-\theta_L }$ and when $\theta_L = 1$, the equation for leveler
-becomes $\dot{\theta_{L}} = - k_L^- v \theta_L.$
+If the surface reaches full coverage, the equations do not naturally prevent the
+coverage rising above full coverage due to the curvature terms. Thus, when
+:math:`\theta_L + \theta_A = 1` then the equation for accelerator becomes
+:math:`\dot{ \theta_A } = -\dot{\theta_L }` and when :math:`\theta_L = 1`, the
+equation for leveler becomes :math:`\dot{\theta_{L}} = - k_L^- v \theta_L.`
 
-The parameters $k_A^+,$ $k_A^-$ and $q$ are both functions of $\eta$
+The parameters :math:`k_A^+`, :math:`k_A^-` and :math:`q` are both functions of :math:`\eta`
 given by, 
 
 .. math:: 
@@ -123,10 +123,11 @@ given by,
    \right) \right)}} + \exp{\left(B_b \left(\eta + V_d \right)
    \right)}
    \\
-   q &= m * \eta + b.
+   q &= m \eta + b.
 
 The following table shows the symbols used in the governing equations
-and their corresponding arguments for the :func:`runLeveler` function.
+and their corresponding arguments for the
+:func:`~examples.levelSet.electroChem.leveler.runLeveler` function.
 
 .. math::
     
@@ -198,19 +199,29 @@ can be obtained by running this example.
 .. image:: electroChem/accelerator.*
    :width: 90%
    :align: center
-   :alt: resulting image
+   :alt: accelerator coverage as a function of time during superfill
 
 .. image:: electroChem/leveler.*
    :width: 90%
    :align: center
-   :alt: resulting image
+   :alt: leveler coverage as a function of time during superfill
     
+.. .. bibmissing:: /documentation/refs.bib
+    :sort:
 """
 __docformat__ = 'restructuredtext'
 
 from fipy import *
 
-def runLeveler(kLeveler=0.018, bulkLevelerConcentration=0.02, cellSize=0.1e-7, rateConstant=0.00026, initialAcceleratorCoverage=0.0, levelerDiffusionCoefficient=5e-10, numberOfSteps=400, displayRate=10, displayViewers=True):
+def runLeveler(kLeveler=0.018, 
+               bulkLevelerConcentration=0.02, 
+               cellSize=0.1e-7, 
+               rateConstant=0.00026, 
+               initialAcceleratorCoverage=0.0, 
+               levelerDiffusionCoefficient=5e-10, 
+               numberOfSteps=400, 
+               displayRate=10, 
+               displayViewers=False):
 
     
     kLevelerConsumption = 0.0005
@@ -253,7 +264,7 @@ def runLeveler(kLeveler=0.018, bulkLevelerConcentration=0.02, cellSize=0.1e-7, r
                       trenchDepth = trenchDepth,
                       boundaryLayerDepth = boundaryLayerDepth,
                       aspectRatio = aspectRatio,
-                      angle = pi * 4. / 180.,
+                      angle = numerix.pi * 4. / 180.,
                       bowWidth = 0.,
                       overBumpRadius = 0.,
                       overBumpWidth = 0.)
@@ -265,7 +276,7 @@ def runLeveler(kLeveler=0.018, bulkLevelerConcentration=0.02, cellSize=0.1e-7, r
         value = -1.,
         narrowBandWidth = narrowBandWidth)
 
-    distanceVar.setValue(1., where=mesh.getElectrolyteMask())
+    distanceVar.setValue(1., where=mesh.electrolyteMask)
     
     distanceVar.calcDistanceFunction(narrowBandWidth = 1e10)
     levelerVar = SurfactantVariable(
@@ -293,13 +304,13 @@ def runLeveler(kLeveler=0.018, bulkLevelerConcentration=0.02, cellSize=0.1e-7, r
         value = bulkMetalConcentration)
 
     def depositionCoeff(alpha, i0):
-        expo = exp(-alpha * etaPrime)
-        return 2 * i0 * (expo - expo * exp(etaPrime))
+        expo = numerix.exp(-alpha * etaPrime)
+        return 2 * i0 * (expo - expo * numerix.exp(etaPrime))
 
     coeffSuppressor = depositionCoeff(alphaSuppressor, i0Suppressor)
     coeffAccelerator = depositionCoeff(alphaAccelerator, i0Accelerator)
 
-    exchangeCurrentDensity = acceleratorVar.getInterfaceVar() * (coeffAccelerator - coeffSuppressor) + coeffSuppressor
+    exchangeCurrentDensity = acceleratorVar.interfaceVar * (coeffAccelerator - coeffSuppressor) + coeffSuppressor
 
     currentDensity = metalVar / bulkMetalConcentration * exchangeCurrentDensity
 
@@ -310,8 +321,8 @@ def runLeveler(kLeveler=0.018, bulkLevelerConcentration=0.02, cellSize=0.1e-7, r
         mesh = mesh,
         value = depositionRateVariable)   
 
-    kAccelerator = rateConstant * exp(-alphaAdsorption * etaPrime)
-    kAcceleratorConsumption =  Bd + A / (exp(Ba * (overpotential + Vd)) + exp(Bb * (overpotential + Vd)))
+    kAccelerator = rateConstant * numerix.exp(-alphaAdsorption * etaPrime)
+    kAcceleratorConsumption =  Bd + A / (numerix.exp(Ba * (overpotential + Vd)) + numerix.exp(Bb * (overpotential + Vd)))
     q = m * overpotential + b
 
     levelerSurfactantEquation = AdsorbingSurfactantEquation(
@@ -321,7 +332,7 @@ def runLeveler(kLeveler=0.018, bulkLevelerConcentration=0.02, cellSize=0.1e-7, r
         rateConstant = kLeveler,
         consumptionCoeff = kLevelerConsumption * depositionRateVariable)
 
-    accVar1 = acceleratorVar.getInterfaceVar()
+    accVar1 = acceleratorVar.interfaceVar
     accVar2 = (accVar1 > 0) * accVar1
     accConsumptionCoeff = kAcceleratorConsumption * (accVar2**(q - 1))
 
@@ -345,7 +356,7 @@ def runLeveler(kLeveler=0.018, bulkLevelerConcentration=0.02, cellSize=0.1e-7, r
         diffusionCoeff = metalDiffusionCoefficient,
         metalIonMolarVolume = atomicVolume)
 
-    metalEquationBCs = FixedValue(mesh.getFacesTop(), bulkMetalConcentration)
+    metalVar.constrain(bulkMetalConcentration, mesh.facesTop)
 
     bulkAcceleratorEquation = buildSurfactantBulkDiffusionEquation(
         bulkVar = bulkAcceleratorVar,
@@ -355,10 +366,8 @@ def runLeveler(kLeveler=0.018, bulkLevelerConcentration=0.02, cellSize=0.1e-7, r
         diffusionCoeff = acceleratorDiffusionCoefficient,
         rateConstant = kAccelerator * siteDensity)
 
-    bulkAcceleratorEquationBCs = (FixedValue(
-        mesh.getFacesTop(),
-        bulkAcceleratorConcentration),)
-
+    bulkAcceleratorVar.constrain(bulkAcceleratorConcentration, mesh.facesTop)
+    
     bulkLevelerEquation = buildSurfactantBulkDiffusionEquation(
         bulkVar = bulkLevelerVar,
         distanceVar = distanceVar,
@@ -366,16 +375,14 @@ def runLeveler(kLeveler=0.018, bulkLevelerConcentration=0.02, cellSize=0.1e-7, r
         diffusionCoeff = levelerDiffusionCoefficient,
         rateConstant = kLeveler * siteDensity)
 
-    bulkLevelerEquationBCs =  (FixedValue(
-        mesh.getFacesTop(),
-        bulkLevelerConcentration),)
-
+    bulkLevelerVar.constrain(bulkLevelerConcentration, mesh.facesTop)
+    
     eqnTuple = ( (advectionEquation, distanceVar, (), None),
                  (levelerSurfactantEquation, levelerVar, (), None),
                  (acceleratorSurfactantEquation, acceleratorVar, (), None),
-                 (metalEquation, metalVar,  metalEquationBCs, None),
-                 (bulkAcceleratorEquation, bulkAcceleratorVar, bulkAcceleratorEquationBCs, None),
-                 (bulkLevelerEquation, bulkLevelerVar, bulkLevelerEquationBCs, None))
+                 (metalEquation, metalVar,  (), None),
+                 (bulkAcceleratorEquation, bulkAcceleratorVar, (), GeneralSolver()),
+                 (bulkLevelerEquation, bulkLevelerVar, (), GeneralSolver()))
 
     levelSetUpdateFrequency = int(0.7 * narrowBandWidth / cellSize / cflNumber / 2)
 
@@ -383,8 +390,8 @@ def runLeveler(kLeveler=0.018, bulkLevelerConcentration=0.02, cellSize=0.1e-7, r
 
     if displayViewers:
         viewers = (
-            MayaviSurfactantViewer(distanceVar, acceleratorVar.getInterfaceVar(), zoomFactor = 1e6, datamax=0.5, datamin=0.0, smooth = 1, title = 'accelerator coverage'),
-            MayaviSurfactantViewer(distanceVar, levelerVar.getInterfaceVar(), zoomFactor = 1e6, datamax=0.5, datamin=0.0, smooth = 1, title = 'leveler coverage'))
+            MayaviSurfactantViewer(distanceVar, acceleratorVar.interfaceVar, zoomFactor = 1e6, datamax=0.5, datamin=0.0, smooth = 1, title = 'accelerator coverage'),
+            MayaviSurfactantViewer(distanceVar, levelerVar.interfaceVar, zoomFactor = 1e6, datamax=0.5, datamin=0.0, smooth = 1, title = 'leveler coverage'))
         
     for step in range(numberOfSteps):
 
@@ -398,21 +405,18 @@ def runLeveler(kLeveler=0.018, bulkLevelerConcentration=0.02, cellSize=0.1e-7, r
 
         extensionVelocityVariable.setValue(depositionRateVariable)
 
-        extOnInt = where(distanceVar > 0,
-                         where(distanceVar < 2 * cellSize,
-                               extensionVelocityVariable,
-                               0),
-                         0)
+        extOnInt = numerix.where(distanceVar > 0,
+                                 numerix.where(distanceVar < 2 * cellSize,
+                                               extensionVelocityVariable,
+                                               0),
+                                 0)
 
         dt = cflNumber * cellSize / extOnInt.max()
 
-        id = nonzero(distanceVar._getInterfaceFlag())[0].max()
+        id = numerix.nonzero(distanceVar._interfaceFlag)[0].max()
         distanceVar.extendVariable(extensionVelocityVariable, deleteIslands = True)
 
-        extensionVelocityVariable[mesh.getFineMesh().getNumberOfCells():] = 0.
-
-        for eqn, var, BCs, solver in eqnTuple:
-            var.updateOld()
+        extensionVelocityVariable[mesh.fineMesh.numberOfCells:] = 0.
 
         for eqn, var, BCs, solver in eqnTuple:
             eqn.solve(var, boundaryConditions = BCs, dt = dt, solver=solver)
