@@ -285,8 +285,8 @@ these issues automatically, so we could just write::
 
     chemPotA = Vm * (enthalpyA * p(phase) + WA * g(phase)) + R * T * log(C)
     chemPotB = Vm * (enthalpyB * p(phase) + WB * g(phase)) + R * T * log(1-C)
-    flux = Mc * (chemPotB - chemPotA).getFaceGrad()
-    eq = TransientTerm() == flux.getDivergence()
+    flux = Mc * (chemPotB - chemPotA).faceGrad
+    eq = TransientTerm() == flux.divergence
 
 Although the second syntax would essentially work as written, such an
 explicit implementation would be very slow. In order to take advantage
@@ -374,12 +374,12 @@ or
 
 >>> Dl = Variable(value=1e-5) # cm**2 / s
 >>> Ds = Variable(value=1e-9) # cm**2 / s
->>> D = (Dl - Ds) * phase.getArithmeticFaceValue() + Dl
+>>> D = (Dl - Ds) * phase.arithmeticFaceValue + Dl
 
 >>> phaseTransformationVelocity = \
-...  ((enthalpyB - enthalpyA) * p(phase).getFaceGrad()
-...   + 0.5 * (WB - WA) * g(phase).getFaceGrad()) \
-...   * D * (1. - C).getHarmonicFaceValue() * Vm / (R * T)
+...  ((enthalpyB - enthalpyA) * p(phase).faceGrad
+...   + 0.5 * (WB - WA) * g(phase).faceGrad) \
+...   * D * (1. - C).harmonicFaceValue * Vm / (R * T)
 
 
 .. index:: PowerLawConvectionTerm
@@ -393,7 +393,7 @@ or
 We initialize the phase field to a step function in the middle of the domain
 
 >>> phase.setValue(1.)
->>> phase.setValue(0., where=mesh.getCellCenters()[0] > L/2.)
+>>> phase.setValue(0., where=mesh.cellCenters[0] > L/2.)
 
 and start with a uniform composition field :math:`C = 1/2`
    
@@ -412,13 +412,13 @@ deduce the liquidus and solidus compositions as
 
 .. index:: exp
    
->>> Cl = (1. - exp(-enthalpyA * Vm / (R * T))) \
-...   / (exp(-enthalpyB * Vm / (R * T)) - exp(-enthalpyA * Vm / (R * T)))
->>> Cs = exp(-enthalpyB * Vm / (R * T)) * Cl
+>>> Cl = (1. - numerix.exp(-enthalpyA * Vm / (R * T))) \
+...   / (numerix.exp(-enthalpyB * Vm / (R * T)) - numerix.exp(-enthalpyA * Vm / (R * T)))
+>>> Cs = numerix.exp(-enthalpyB * Vm / (R * T)) * Cl
 
 The phase fraction is predicted by the lever rule
 
->>> Cavg = C.getCellVolumeAverage()
+>>> Cavg = C.cellVolumeAverage
 >>> fraction = (Cl - Cavg) / (Cl - Cs)
 
 For the special case of ``fraction = Cavg = 0.5``, a little bit of algebra
@@ -446,8 +446,12 @@ We'll need a function to return the two conditions for equilibrium
 .. index:: log, array
 
 >>> def equilibrium(C):
-...     return [array(enthalpyA * Vm + R * T * log(1 - C[0]) - R * T * log(1 - C[1])),
-...             array(enthalpyB * Vm + R * T * log(C[0]) - R * T * log(C[1]))]
+...     return [numerix.array(enthalpyA * Vm 
+...                           + R * T * numerix.log(1 - C[0]) 
+...                           - R * T * numerix.log(1 - C[1])),
+...             numerix.array(enthalpyB * Vm 
+...                           + R * T * numerix.log(C[0]) 
+...                           - R * T * numerix.log(C[1]))]
                
 and we'll have much better luck if we also supply the Jacobian
 
@@ -466,34 +470,34 @@ and we'll have much better luck if we also supply the Jacobian
    \end{matrix}\right]
    
 >>> def equilibriumJacobian(C):
-...     return R * T * array([[-1. / (1 - C[0]), 1. / (1 - C[1])],
-...                           [ 1. / C[0],      -1. / C[1]]])
+...     return R * T * numerix.array([[-1. / (1 - C[0]), 1. / (1 - C[1])],
+...                                   [ 1. / C[0],      -1. / C[1]]])
 
 .. index:: SciPy
 
 >>> try:
-...     from scipy.optimize import fsolve
+...     from scipy.optimize import fsolve # doctest: +SCIPY
 ...     CsRoot, ClRoot = fsolve(func=equilibrium, x0=[0.5, 0.5], 
-...                             fprime=equilibriumJacobian)
+...                             fprime=equilibriumJacobian) # doctest: +SCIPY
 ... except ImportError:
 ...     ClRoot = CsRoot = 0
 ...     print "The SciPy library is not available to calculate the solidus and \
 ... liquidus concentrations"
 
->>> print Cl.allclose(ClRoot)
+>>> print Cl.allclose(ClRoot) # doctest: +SCIPY
 1
->>> print Cs.allclose(CsRoot)
+>>> print Cs.allclose(CsRoot) # doctest: +SCIPY
 1
 
 We plot the result against the sharp interface solution
 
 >>> sharp = CellVariable(name="sharp", mesh=mesh)
->>> x = mesh.getCellCenters()[0]
+>>> x = mesh.cellCenters[0]
 >>> sharp.setValue(Cs, where=x < L * fraction)
 >>> sharp.setValue(Cl, where=x >= L * fraction)
 
 .. index::
-   module: viewers
+   module: fipy.viewers
 
 >>> if __name__ == '__main__':
 ...     viewer = Viewer(vars=(phase, C, sharp), 
@@ -516,10 +520,10 @@ equations) as a test for how long to sweep. Because of the
 and cannot be solved by the default :class:`~fipy.solvers.pysparse.linearPCGSolver.LinearPCGSolver`. Therefore, we use a
 :class:`~fipy.solvers.DefaultAsymmetricSolver` for this equation.
 
-.. index:: DefaultAsymmetricSolver, solve, sweep
+.. .. index:: DefaultAsymmetricSolver, solve, sweep
 
-We now use the ":meth:`~fipy.terms.Term.sweep`" method instead of ":meth:`~fipy.terms.Term.solve`" because we
-require the residual.
+We now use the ":meth:`~fipy.terms.term.Term.sweep`" method instead of
+":meth:`~fipy.terms.term.Term.solve`" because we require the residual.
 
 >>> solver = DefaultAsymmetricSolver(tolerance=1e-10)
 
@@ -537,19 +541,20 @@ require the residual.
 .. image:: binary/stationary.*
    :width: 90%
    :align: center
+   :alt: phase and composition fields in equilibrium, compared with phase diagram concentrations
 
 We verify that the bulk phases have shifted to the predicted solidus and
 liquidus compositions
 
->>> X = mesh.getFaceCenters()[0]
->>> print Cs.allclose(C.getFaceValue()[X==0], atol=2e-4)
+>>> X = mesh.faceCenters[0]
+>>> print Cs.allclose(C.faceValue[X.value==0], atol=2e-4)
 True
->>> print Cl.allclose(C.getFaceValue()[X==L], atol=2e-4)
+>>> print Cl.allclose(C.faceValue[X.value==L], atol=2e-4)
 True
 
 and that the phase fraction remains unchanged
 
->>> print fraction.allclose(phase.getCellVolumeAverage(), atol=2e-4)
+>>> print fraction.allclose(phase.cellVolumeAverage, atol=2e-4)
 1
 
 while conserving mass overall
@@ -572,7 +577,12 @@ diffusion and of phase transformation compete with each other).
 
 >>> dt = 1.e-6
 
->>> for i in range(100):
+>>> if __name__ == '__main__':
+...     timesteps = 100
+... else:
+...     timesteps = 10
+
+>>> for i in range(timesteps):
 ...     phase.updateOld()
 ...     C.updateOld()
 ...     phaseRes = 1e+10
@@ -589,6 +599,7 @@ diffusion and of phase transformation compete with each other).
 .. image:: binary/moving.*
    :width: 90%
    :align: center
+   :alt: phase and composition fields in during solidification, compared with final phase diagram concentrations
 
 We see that the composition on either side of the interface approach the
 sharp-interface solidus and liquidus, but it will take a great many more
@@ -607,7 +618,8 @@ expected values.
    examine different temperatures in this example, so we declare :math:`T` 
    as a :class:`~fipy.variables.variable.Variable`
 
-
+.. .. bibmissing:: /documentation/refs.bib
+    :sort:
 """
 
 __docformat__ = 'restructuredtext'
