@@ -95,17 +95,19 @@ recommended values from CODATA_. Other conversion factors
 """
 __docformat__ = 'restructuredtext'
 
-import re, string
+import re
 
 from fipy.tools import numerix
 from fipy.tools.numerix import MA
 from fipy.tools.numerix import umath
+from fipy.tools.decorators import getsetDeprecated
 
-from NumberDict import _NumberDict
+from fipy.tools.dimensions.NumberDict import _NumberDict
+
+__all__ = ["PhysicalField", "PhysicalUnit"]
 
 # Class definitions
 
-
 class PhysicalField(object):
     """
     Physical field or quantity with units
@@ -168,7 +170,9 @@ class PhysicalField(object):
         .. _PhysicalQuantity: http://starship.python.net/~hinsen/ScientificPython/ScientificPythonManual/Scientific_31.html
         .. _Numeric: http://www.numpy.org
         """
-        if hasattr(value, "getValue") and callable(value.getValue):
+        if hasattr(value, "value") and not isinstance(value, PhysicalField):
+            value = value.value
+        elif hasattr(value, "getValue") and callable(value.getValue):
             value = value.getValue()
             
         if isinstance(value, PhysicalField):
@@ -186,7 +190,7 @@ class PhysicalField(object):
         elif unit is not None:
             unit = _findUnit(unit)
         elif type(value) is type(''):
-            s = string.strip(value)
+            s = value.strip()
             match = PhysicalField._number.match(s)
             if match is None:
                 value = 1
@@ -213,13 +217,16 @@ class PhysicalField(object):
 ##             unit = _findUnit("")
 
         self.value = numerix.array(value)
-        self.unit = unit
+        self._unit = unit
         if array is not None:
             array[:] = self.value
             self.value = array
 
     _number = re.compile('[+-]?[0-9]+(\\.[0-9]*)?([eE][+-]?[0-9]+)?')
 
+    def ravel(self):
+        return self.value.ravel()
+
     def copy(self):
         """
         Make a duplicate.
@@ -277,7 +284,7 @@ class PhysicalField(object):
             return value
         else:
             return (self.__class__.__name__ + '(' + value + ',' + 
-                    `self.unit.name()` + ')')
+                    repr(self.unit.name()) + ')')
 
     def tostring(self, max_line_width=75, precision=8, suppress_small=False, separator=' '):
         """
@@ -387,7 +394,7 @@ class PhysicalField(object):
     __rmul__ = __mul__
     multiply = __mul__
 
-    def __div__(self, other):
+    def __truediv__(self, other):
         """
         Divide two physical quantities.  The unit of the result is the
         unit of the first operand divided by the unit of the second.
@@ -408,7 +415,7 @@ class PhysicalField(object):
         .. _Numeric: http://www.numpy.org
         """
         if _isVariable(other):
-            return other.__rdiv__(self)
+            return other.__rtruediv__(self)
         if type(other) is type(''):
             other = self.__class__(value = other)
         if not isinstance(other,PhysicalField):
@@ -422,11 +429,12 @@ class PhysicalField(object):
         else:
             return self.__class__(value = value, unit = unit)
 
-    divide = __div__
+    __div__ = __truediv__
+    divide = __truediv__
     
-    def __rdiv__(self, other):
+    def __rtruediv__(self, other):
         if _isVariable(other):
-            return other.__div__(self)
+            return other.__truediv__(self)
         if type(other) is type(''):
             other = PhysicalField(value = other)
         if not isinstance(other,PhysicalField):
@@ -440,6 +448,8 @@ class PhysicalField(object):
         else:
             return self.__class__(value = value, unit = unit)
 
+    __rdiv__ = __rtruediv__
+    
     def __mod__(self, other):
         """
         Return the remainder of dividing two physical quantities.  The unit of the result is the
@@ -521,7 +531,7 @@ class PhysicalField(object):
         
     def _inMyUnits(self, other):
         if _isVariable(other):
-            other = other.getValue()
+            other = other.value
             
         if not isinstance(other, PhysicalField):
             if type(other) is type(''):
@@ -571,8 +581,8 @@ class PhysicalField(object):
         
             >>> a = PhysicalField(4.,"m")
             >>> a.itemset(PhysicalField("6 ft"))
-            >>> print a
-            1.8288 m
+            >>> print a.allclose("1.8288 m")
+            1
             >>> a = PhysicalField(((3.,4.),(5.,6.)),"m")
             >>> a.itemset(PhysicalField("6 ft"))
             Traceback (most recent call last):
@@ -606,18 +616,19 @@ class PhysicalField(object):
         Required to prevent numpy not calling the reverse binary operations.
         Both the following tests are examples ufuncs.
         
-           >>> print type(numerix.array([1.0, 2.0]) * PhysicalField([1.0, 2.0], unit="m"))
-           <class 'fipy.tools.dimensions.physicalField.PhysicalField'>
+        >>> from fipy.tools.dimensions.physicalField import PhysicalField
+        >>> print type(numerix.array([1.0, 2.0]) * PhysicalField([1.0, 2.0], unit="m"))
+        <class 'fipy.tools.dimensions.physicalField.PhysicalField'>
 
         For not very intelligible reasons, the `PhysicalField`\ness gets cast 
         away if there are no units. Probably not harmful, so not worth investigating
         
-           >>> print type(numerix.array([1.0, 2.0]) * PhysicalField([1.0, 2.0]))
-           <type 'numpy.ndarray'>
+        >>> print type(numerix.array([1.0, 2.0]) * PhysicalField([1.0, 2.0])) is type(numerix.array(1))
+        1
 
-           >>> from scipy.special import gamma as Gamma
-           >>> print type(Gamma(PhysicalField([1.0, 2.0])))
-           <type 'numpy.ndarray'>
+        >>> from scipy.special import gamma as Gamma # doctest: +SCIPY
+        >>> print type(Gamma(PhysicalField([1.0, 2.0]))) is type(numerix.array(1)) # doctest: +SCIPY
+        1
         """
         result = arr
         
@@ -668,16 +679,21 @@ class PhysicalField(object):
         return numerix.array(value, t)
         
 #         if self.unit.isDimensionlessOrAngle():
-#             value = self.getNumericValue()
+#             value = self.numericValue
 # ##            if type(value) is type(numerix.array((0))) and (t is None or t == value.typecode()):
 #             if type(value) is type(numerix.array((0))) and (t is None or t == value.dtype.char):
 #                 return value
 #             else:
-#                 return numerix.array(self.getNumericValue(), t)
+#                 return numerix.array(self.numericValue, t)
 #         else:
 #             raise TypeError, 'Numeric array value must be dimensionless'
         
+    @getsetDeprecated
     def _getArray(self):
+        return self._array
+
+    @property
+    def _array(self):
         if self.unit.isDimensionlessOrAngle():
             return self.value
         else:
@@ -693,7 +709,7 @@ class PhysicalField(object):
         As a special case, quantities with angular units are converted to
         base units (radians) and then assumed dimensionless.
         
-            >>> print round(float(PhysicalField("2. deg")), 6)
+            >>> print numerix.round_(float(PhysicalField("2. deg")), 6)
             0.034907
          
         If the quantity is not dimensionless, the conversion fails.
@@ -713,7 +729,7 @@ class PhysicalField(object):
         .. _Numeric: http://www.numpy.org
         """
         if self.unit.isDimensionlessOrAngle():
-            return float(self.getNumericValue())
+            return float(self.numericValue)
         else:
             raise TypeError, 'Not possible to convert a PhysicalField with dimensions to float'
                     
@@ -749,7 +765,7 @@ class PhysicalField(object):
         
         And so must array dimensions
         
-            >>> print a > PhysicalField(((3.,13.,4.),(17.,6.,2.)),"ft")
+            >>> print a > PhysicalField(((3.,13.,4.),(17.,6.,2.)),"ft") #doctest: +IGNORE_EXCEPTION_DETAIL
             Traceback (most recent call last):
                 ...
             ValueError: shape mismatch: objects cannot be broadcast to a single shape
@@ -769,6 +785,8 @@ class PhysicalField(object):
         other = self._inMyUnits(other)
         return self.value == other.value
         
+    __hash__ = object.__hash__
+        
     def __ne__(self,other):
         other = self._inMyUnits(other)
         return self.value != other.value
@@ -797,7 +815,7 @@ class PhysicalField(object):
         """
         unit = _findUnit(unit)
         self.value = _convertValue (self.value, self.unit, unit)
-        self.unit = unit
+        self._unit = unit
 
     def inRadians(self):
         """
@@ -859,9 +877,9 @@ class PhysicalField(object):
         the unit of the object.  If one unit is specified, the return value
         is a single `PhysicalField`.
         
-            >>> freeze = PhysicalField('0 degC')
-            >>> print freeze.inUnitsOf('degF')
-            32.0 degF
+        >>> freeze = PhysicalField('0 degC')
+        >>> print freeze.inUnitsOf('degF').allclose("32.0 degF")
+        1
         
         If several units are specified, the return value is a tuple of
 	`PhysicalField` instances with with one element per unit such that
@@ -870,9 +888,11 @@ class PhysicalField(object):
         This is used to convert to irregular unit systems like
         hour/minute/second.  The original object will not be changed.
         
-            >>> t = PhysicalField(314159., 's')
-            >>> [str(element) for element in t.inUnitsOf('d','h','min','s')]
-            ['3.0 d', '15.0 h', '15.0 min', '59.0 s']
+        >>> t = PhysicalField(314159., 's')
+        >>> print numerix.allclose([e.allclose(v) for (e, v) in zip(t.inUnitsOf('d','h','min','s'),
+        ...                                                         ['3.0 d', '15.0 h', '15.0 min', '59.0 s'])], 
+        ...                        True)
+        1
         """
         units = map(_findUnit, units)
         if len(units) == 1:
@@ -910,36 +930,51 @@ class PhysicalField(object):
         """
         
         if not hasattr(self, 'typecode'):
-            self.typecode = numerix.obj2sctype(rep=numerix.array(self.getNumericValue()), 
+            self.typecode = numerix.obj2sctype(rep=numerix.array(self.numericValue), 
                                                default=default)
         
         return self.typecode
 
+    @getsetDeprecated
     def getUnit(self):
+        return self.unit
+
+    def _getUnit(self):
         """
         Return the unit object of `self`.
         
-            >>> PhysicalField("1 m").getUnit()
+            >>> PhysicalField("1 m").unit
             <PhysicalUnit m>
         """
-        return self.unit
+        return self._unit
         
+    @getsetDeprecated
     def setUnit(self, unit):
+        self.unit = unit
+
+    def _setUnit(self, unit):
         """
         Change the unit object of `self` to `unit`
         
             >>> a = PhysicalField(value="1 m")
-            >>> a.setUnit("m**2/s")
+            >>> a.unit = ("m**2/s")
             >>> print a
             1.0 m**2/s
         """
-        self.unit = _findUnit(unit)
+        self._unit = _findUnit(unit)
+
+    unit = property(_getUnit, _setUnit)
         
+    @getsetDeprecated
     def getNumericValue(self):
+        return self.numericValue
+
+    @property
+    def numericValue(self):
         """
         Return the `PhysicalField` without units, after conversion to base SI units.
         
-            >>> print round(PhysicalField("1 inch").getNumericValue(), 6)
+            >>> print numerix.round_(PhysicalField("1 inch").numericValue, 6)
             0.0254
         """
         return self.inSIUnits().value
@@ -950,8 +985,8 @@ class PhysicalField(object):
         Return the quantity with all units reduced to their base SI elements.
         
             >>> e = PhysicalField('2.7 Hartree*Nav')
-            >>> print e.inBaseUnits()
-            7088849.01085 kg*m**2/s**2/mol
+            >>> print e.inBaseUnits().allclose("7088849.01085 kg*m**2/s**2/mol")
+            1
         """
         if self.unit.factor != 1:
             new_value = self.value * self.unit.factor
@@ -981,8 +1016,8 @@ class PhysicalField(object):
         Return the quantity with all units reduced to SI-compatible elements.
         
             >>> e = PhysicalField('2.7 Hartree*Nav')
-            >>> print e.inSIUnits()
-            7088849.01085 kg*m**2/s**2/mol
+            >>> print e.inSIUnits().allclose("7088849.01085 kg*m**2/s**2/mol")
+            1
         """
         if self.unit.factor != 1:
             return self.inBaseUnits()
@@ -997,12 +1032,12 @@ class PhysicalField(object):
         """
         Return the inverse cosine of the `PhysicalField` in radians
         
-            >>> print PhysicalField(0).arccos()
-            1.57079632679 rad
+            >>> print PhysicalField(0).arccos().allclose("1.57079632679 rad")
+            1
         
         The input `PhysicalField` must be dimensionless
         
-            >>> print round(PhysicalField("1 m").arccos(), 6)
+            >>> print numerix.round_(PhysicalField("1 m").arccos(), 6)
             Traceback (most recent call last):
                 ...
             TypeError: Incompatible units
@@ -1013,12 +1048,13 @@ class PhysicalField(object):
         """
         Return the inverse hyperbolic cosine of the `PhysicalField`
         
-            >>> print PhysicalField(2).arccosh()
-            1.31695789692
+            >>> print numerix.allclose(PhysicalField(2).arccosh(),
+            ...                        1.31695789692)
+            1
         
         The input `PhysicalField` must be dimensionless
         
-            >>> print round(PhysicalField("1. m").arccosh(), 6)
+            >>> print numerix.round_(PhysicalField("1. m").arccosh(), 6)
             Traceback (most recent call last):
                 ...
             TypeError: Incompatible units
@@ -1029,12 +1065,12 @@ class PhysicalField(object):
         """
         Return the inverse sine of the `PhysicalField` in radians
         
-            >>> print PhysicalField(1).arcsin()
-            1.57079632679 rad
+            >>> print PhysicalField(1).arcsin().allclose("1.57079632679 rad")
+            1
         
         The input `PhysicalField` must be dimensionless
         
-            >>> print round(PhysicalField("1 m").arcsin(), 6)
+            >>> print numerix.round_(PhysicalField("1 m").arcsin(), 6)
             Traceback (most recent call last):
                 ...
             TypeError: Incompatible units
@@ -1095,9 +1131,9 @@ class PhysicalField(object):
         """
         Return the cosine of the `PhysicalField`
         
-            >>> print round(PhysicalField(2*numerix.pi/6,"rad").cos(), 6)
+            >>> print numerix.round_(PhysicalField(2*numerix.pi/6,"rad").cos(), 6)
             0.5
-            >>> print round(PhysicalField(60.,"deg").cos(), 6)
+            >>> print numerix.round_(PhysicalField(60.,"deg").cos(), 6)
             0.5
         
         The units of the `PhysicalField` must be an angle
@@ -1129,9 +1165,9 @@ class PhysicalField(object):
         """
         Return the tangent of the `PhysicalField`
         
-            >>> round(PhysicalField(numerix.pi/4,"rad").tan(), 6)
+            >>> numerix.round_(PhysicalField(numerix.pi/4,"rad").tan(), 6)
             1.0
-            >>> round(PhysicalField(45,"deg").tan(), 6)
+            >>> numerix.round_(PhysicalField(45,"deg").tan(), 6)
             1.0
         
         The units of the `PhysicalField` must be an angle
@@ -1163,15 +1199,15 @@ class PhysicalField(object):
         """
         Return the arctangent of `self` divided by `other` in radians
         
-            >>> print round(PhysicalField(2.).arctan2(PhysicalField(5.)), 6)
+            >>> print numerix.round_(PhysicalField(2.).arctan2(PhysicalField(5.)), 6)
             0.380506
         
         The input `PhysicalField` objects must be in the same dimensions
         
-            >>> print round(PhysicalField(2.54, "cm").arctan2(PhysicalField(1., "inch")), 6)
+            >>> print numerix.round_(PhysicalField(2.54, "cm").arctan2(PhysicalField(1., "inch")), 6)
             0.785398
 
-            >>> print round(PhysicalField(2.).arctan2(PhysicalField("5. m")), 6)
+            >>> print numerix.round_(PhysicalField(2.).arctan2(PhysicalField("5. m")), 6)
             Traceback (most recent call last):
                 ...
             TypeError: Incompatible units
@@ -1183,12 +1219,12 @@ class PhysicalField(object):
         """
         Return the arctangent of the `PhysicalField` in radians
         
-            >>> print round(PhysicalField(1).arctan(), 6)
+            >>> print numerix.round_(PhysicalField(1).arctan(), 6)
             0.785398
         
         The input `PhysicalField` must be dimensionless
         
-            >>> print round(PhysicalField("1 m").arctan(), 6)
+            >>> print numerix.round_(PhysicalField("1 m").arctan(), 6)
             Traceback (most recent call last):
                 ...
             TypeError: Incompatible units
@@ -1204,7 +1240,7 @@ class PhysicalField(object):
         
         The input `PhysicalField` must be dimensionless
         
-            >>> print round(PhysicalField("1 m").arctanh(), 6)
+            >>> print numerix.round_(PhysicalField("1 m").arctanh(), 6)
             Traceback (most recent call last):
                 ...
             TypeError: Incompatible units
@@ -1216,12 +1252,12 @@ class PhysicalField(object):
         """
         Return the natural logarithm of the `PhysicalField`
         
-            >>> print round(PhysicalField(10).log(), 6)
+            >>> print numerix.round_(PhysicalField(10).log(), 6)
             2.302585
             
         The input `PhysicalField` must be dimensionless
         
-            >>> print round(PhysicalField("1. m").log(), 6)
+            >>> print numerix.round_(PhysicalField("1. m").log(), 6)
             Traceback (most recent call last):
                 ...
             TypeError: Incompatible units
@@ -1232,12 +1268,12 @@ class PhysicalField(object):
         """
         Return the base-10 logarithm of the `PhysicalField`
         
-            >>> print round(PhysicalField(10.).log10(), 6)
+            >>> print numerix.round_(PhysicalField(10.).log10(), 6)
             1.0
             
         The input `PhysicalField` must be dimensionless
         
-            >>> print round(PhysicalField("1. m").log10(), 6)
+            >>> print numerix.round_(PhysicalField("1. m").log10(), 6)
             Traceback (most recent call last):
                 ...
             TypeError: Incompatible units
@@ -1332,12 +1368,20 @@ class PhysicalField(object):
         """
         numerix.put(self.value, indices, self._inMyUnits(values).value)
       
+    @getsetDeprecated
     def getShape(self):
+        return self.shape
+
+    @property
+    def shape(self):
+        """Tuple of array dimensions."""
         from fipy.tools import numerix
         return numerix.getShape(self.value)
         
-    shape = property(fget=getShape, doc="Tuple of array dimensions.")
-
+    @property
+    def itemsize(self):
+        return self.value.itemsize
+        
     def reshape(self, shape):
         """
         Changes the shape of `self` to that specified in `shape`
@@ -1379,7 +1423,7 @@ class PhysicalField(object):
         """
         other = self._inMyUnits(other)
         if atol is None:
-            atol = PhysicalField(1.e-5, self.getUnit())
+            atol = PhysicalField(1.e-5, self.unit)
         else:
             atol = self._inMyUnits(atol)
             
@@ -1391,7 +1435,7 @@ class PhysicalField(object):
         """
         other = self._inMyUnits(other)
         return MA.allequal(self.value, other.value)
-        
+
 class PhysicalUnit:
     """
     A `PhysicalUnit` represents the units of a `PhysicalField`.
@@ -1432,32 +1476,55 @@ class PhysicalUnit:
 
     __str__ = __repr__
 
-    def __cmp__(self, other):
-        """
-        Determine if units are identical
-        
-            >>> a = PhysicalField("1. m")
-            >>> b = PhysicalField("3. ft")
-            >>> a.getUnit() == b.getUnit()
-            0
-            >>> a.getUnit() == b.inBaseUnits().getUnit()
-            1
-            
-        Units can only be compared with other units
-        
-            >>> a.getUnit() == 3
-            Traceback (most recent call last):
-                ...
-            TypeError: PhysicalUnits can only be compared with other PhysicalUnits
-        """
-        if not isinstance(other,PhysicalUnit):
+    def _checkSame(self, other):
+        if not isinstance(other, PhysicalUnit):
             if other == 1:
                 return self.isDimensionless()
             else:
                 raise TypeError, 'PhysicalUnits can only be compared with other PhysicalUnits'
         if not numerix.alltrue(self.powers == other.powers):
             raise TypeError, 'Incompatible units'
-        return cmp(self.factor, other.factor)
+
+    def __eq__(self, other):
+        """
+        Determine if units are identical
+        
+        >>> a = PhysicalField("1. m")
+        >>> b = PhysicalField("3. ft")
+        >>> a.unit == b.unit
+        0
+        >>> a.unit == b.inBaseUnits().unit
+        1
+            
+        Units can only be compared with other units
+        
+        >>> a.unit == 3
+        Traceback (most recent call last):
+            ...
+        TypeError: PhysicalUnits can only be compared with other PhysicalUnits
+        """
+        self._checkSame(other)
+        return self.factor == other.factor
+
+    def __lt__(self, other):
+        self._checkSame(other)
+        return self.factor < other.factor
+
+    def __le__(self, other):
+        self._checkSame(other)
+        return self.factor <= other.factor
+
+    def __ne__(self, other):
+        self._checkSame(other)
+        return self.factor != other.factor
+
+    def __gt__(self, other):
+        self._checkSame(other)
+        return self.factor > other.factor
+
+    def __ge__(self, other):
+        self._checkSame(other)
+        return self.factor >= other.factor
 
     def __mul__(self, other):
         """
@@ -1465,31 +1532,31 @@ class PhysicalUnit:
         
             >>> a = PhysicalField("1. m")
             >>> b = PhysicalField("3. ft")
-            >>> a.getUnit() * b.getUnit()
+            >>> a.unit * b.unit
             <PhysicalUnit ft*m>
-            >>> a.getUnit() * b.inBaseUnits().getUnit()
+            >>> a.unit * b.inBaseUnits().unit
             <PhysicalUnit m**2>
             >>> c = PhysicalField("1. s")
             >>> d = PhysicalField("3. Hz")
-            >>> c.getUnit() * d.getUnit()
+            >>> c.unit * d.unit
             <PhysicalUnit Hz*s>
-            >>> c.getUnit() * d.inBaseUnits().getUnit()
+            >>> c.unit * d.inBaseUnits().unit
             <PhysicalUnit 1>
             
         or multiply units by numbers
         
-            >>> a.getUnit() * 3.
+            >>> a.unit * 3.
             <PhysicalUnit m*3.0>
         
         Units must have zero offset to be multiplied
         
             >>> e = PhysicalField("1. kB")
             >>> f = PhysicalField("25. degC")
-            >>> e.getUnit() * f.getUnit()
+            >>> e.unit * f.unit
             Traceback (most recent call last):
                 ...
             TypeError: cannot multiply units with non-zero offset
-            >>> e.getUnit() * f.inBaseUnits().getUnit()
+            >>> e.unit * f.inBaseUnits().unit
             <PhysicalUnit kB*K>
         """
         if self.offset != 0 or (isinstance(other,PhysicalUnit) and other.offset != 0):
@@ -1506,37 +1573,37 @@ class PhysicalUnit:
 
     __rmul__ = __mul__
     
-    def __div__(self, other):
+    def __truediv__(self, other):
         """
         Divide one unit by another
         
             >>> a = PhysicalField("1. m")
             >>> b = PhysicalField("3. ft")
-            >>> a.getUnit() / b.getUnit()
+            >>> a.unit / b.unit
             <PhysicalUnit m/ft>
-            >>> a.getUnit() / b.inBaseUnits().getUnit()
+            >>> a.unit / b.inBaseUnits().unit
             <PhysicalUnit 1>
             >>> c = PhysicalField("1. s")
             >>> d = PhysicalField("3. Hz")
-            >>> c.getUnit() / d.getUnit()
+            >>> c.unit / d.unit
             <PhysicalUnit s/Hz>
-            >>> c.getUnit() / d.inBaseUnits().getUnit()
+            >>> c.unit / d.inBaseUnits().unit
             <PhysicalUnit s**2/1>
             
         or divide units by numbers
         
-            >>> a.getUnit() / 3.
+            >>> a.unit / 3.
             <PhysicalUnit m/3.0>
         
         Units must have zero offset to be divided
         
             >>> e = PhysicalField("1. J")
             >>> f = PhysicalField("25. degC")
-            >>> e.getUnit() / f.getUnit()
+            >>> e.unit / f.unit
             Traceback (most recent call last):
                 ...
             TypeError: cannot divide units with non-zero offset
-            >>> e.getUnit() / f.inBaseUnits().getUnit()
+            >>> e.unit / f.inBaseUnits().unit
             <PhysicalUnit J/K>
         """
         if self.offset != 0 or (isinstance(other,PhysicalUnit) and other.offset != 0):
@@ -1549,22 +1616,24 @@ class PhysicalUnit:
             return PhysicalUnit(self.names+{str(other): -1},
                                 self.factor/other, self.powers)
 
-    def __rdiv__(self, other):
+    __div__ = __truediv__
+    
+    def __rtruediv__(self, other):
         """
         Divide something by a unit
         
             >>> a = PhysicalField("1. m")
-            >>> 3. / a.getUnit()
+            >>> 3. / a.unit
             <PhysicalUnit 3.0/m>
         
         Units must have zero offset to be divided
         
             >>> b = PhysicalField("25. degC")
-            >>> 3. / b.getUnit()
+            >>> 3. / b.unit
             Traceback (most recent call last):
                 ...
             TypeError: cannot divide units with non-zero offset
-            >>> 3. / b.inBaseUnits().getUnit()
+            >>> 3. / b.inBaseUnits().unit
             <PhysicalUnit 3.0/K>
         """
         if self.offset != 0 or (isinstance(other,PhysicalUnit) and other.offset != 0):
@@ -1578,20 +1647,22 @@ class PhysicalUnit:
             return PhysicalUnit({str(other): 1}-self.names,
                                 other/self.factor,
                                 -self.powers)
+                                
+    __rdiv__ = __rtruediv__
 
     def __pow__(self, other):
         """
         Raise a unit to an integer power
         
             >>> a = PhysicalField("1. m")
-            >>> a.getUnit()**2
+            >>> a.unit**2
             <PhysicalUnit m**2>
-            >>> a.getUnit()**-2
+            >>> a.unit**-2
             <PhysicalUnit 1/m**2>
             
         Non-integer powers are not supported
         
-            >>> a.getUnit()**0.5
+            >>> a.unit**0.5
             Traceback (most recent call last):
                 ...
             TypeError: Illegal exponent
@@ -1599,11 +1670,11 @@ class PhysicalUnit:
         Units must have zero offset to be exponentiated
         
             >>> b = PhysicalField("25. degC")
-            >>> b.getUnit()**2
+            >>> b.unit**2
             Traceback (most recent call last):
                 ...
             TypeError: cannot exponentiate units with non-zero offset
-            >>> b.inBaseUnits().getUnit()**2
+            >>> b.inBaseUnits().unit**2
             <PhysicalUnit K**2>
         """
         if self.offset != 0:
@@ -1625,11 +1696,11 @@ class PhysicalUnit:
         if abs(inv_exp-rounded) < 1.e-10:
             if numerix.logical_and.reduce(self.powers % rounded == 0):
                 f = pow(self.factor, other)
-                p = self.powers / rounded
+                p = self.powers // rounded
                 if reduce(lambda a, b: a and b,
                           map(lambda x, e=rounded: x%e == 0,
                               self.names.values())):
-                    names = self.names/rounded
+                    names = self.names // rounded
                 else:
                     names = _NumberDict()
                     if f != 1.:
@@ -1647,13 +1718,13 @@ class PhysicalUnit:
         
             >>> a = PhysicalField("1. mm")
             >>> b = PhysicalField("1. inch")
-            >>> print round(b.getUnit().conversionFactorTo(a.getUnit()), 6)
+            >>> print numerix.round_(b.unit.conversionFactorTo(a.unit), 6)
             25.4
             
         Units must have the same fundamental SI units
         
             >>> c = PhysicalField("1. K")
-            >>> c.getUnit().conversionFactorTo(a.getUnit())
+            >>> c.unit.conversionFactorTo(a.unit)
             Traceback (most recent call last):
                 ...
             TypeError: Incompatible units
@@ -1661,10 +1732,10 @@ class PhysicalUnit:
         If units have different offsets, they must have the same factor
         
             >>> d = PhysicalField("1. degC")
-            >>> c.getUnit().conversionFactorTo(d.getUnit())
+            >>> c.unit.conversionFactorTo(d.unit)
             1.0
             >>> e = PhysicalField("1. degF")
-            >>> c.getUnit().conversionFactorTo(e.getUnit())
+            >>> c.unit.conversionFactorTo(e.unit)
             Traceback (most recent call last):
                 ...
             TypeError: Unit conversion (K to degF) cannot be expressed as a simple multiplicative factor
@@ -1685,9 +1756,9 @@ class PhysicalUnit:
         """
         Return a `tuple` of the multiplication factor and offset between two physical units
         
-            >>> a = PhysicalField("1. K").getUnit()
-            >>> b = PhysicalField("1. degF").getUnit()
-            >>> [str(round(element,6)) for element in b.conversionTupleTo(a)]
+            >>> a = PhysicalField("1. K").unit
+            >>> b = PhysicalField("1. degF").unit
+            >>> [str(numerix.round_(element,6)) for element in b.conversionTupleTo(a)]
             ['0.555556', '459.67']
         """
         if not numerix.alltrue(self.powers == other.powers):
@@ -1719,11 +1790,11 @@ class PhysicalUnit:
             
             >>> a = PhysicalField("1. mm")
             >>> b = PhysicalField("1. inch")
-            >>> print numerix.allclose(a.getUnit().isCompatible(b.getUnit()),
+            >>> print numerix.allclose(a.unit.isCompatible(b.unit),
             ...                        [True, True, True, True, True, True, True, True, True])
             True
             >>> c = PhysicalField("1. K")
-            >>> print numerix.allclose(a.getUnit().isCompatible(c.getUnit()),
+            >>> print numerix.allclose(a.unit.isCompatible(c.unit),
             ...                        [False, True, True, True, False, True, True, True, True])
             True
             
@@ -1734,9 +1805,9 @@ class PhysicalUnit:
         """
         Returns `True` if the unit is dimensionless
             
-            >>> PhysicalField("1. m/m").getUnit().isDimensionless()
+            >>> PhysicalField("1. m/m").unit.isDimensionless()
             1
-            >>> PhysicalField("1. inch").getUnit().isDimensionless()
+            >>> PhysicalField("1. inch").unit.isDimensionless()
             0
         """
         return not numerix.logical_or.reduce(self.powers)
@@ -1745,11 +1816,11 @@ class PhysicalUnit:
         """
         Returns `True` if the unit is an angle
             
-            >>> PhysicalField("1. deg").getUnit().isAngle()
+            >>> PhysicalField("1. deg").unit.isAngle()
             1
-            >>> PhysicalField("1. rad").getUnit().isAngle()
+            >>> PhysicalField("1. rad").unit.isAngle()
             1
-            >>> PhysicalField("1. inch").getUnit().isAngle()
+            >>> PhysicalField("1. inch").unit.isAngle()
             0
         """
         return self.powers[7] == 1 and \
@@ -1759,11 +1830,11 @@ class PhysicalUnit:
         """
         Returns `True` if the 1 divided by the unit is an angle
             
-            >>> PhysicalField("1. deg**-1").getUnit().isInverseAngle()
+            >>> PhysicalField("1. deg**-1").unit.isInverseAngle()
             1
-            >>> PhysicalField("1. 1/rad").getUnit().isInverseAngle()
+            >>> PhysicalField("1. 1/rad").unit.isInverseAngle()
             1
-            >>> PhysicalField("1. inch").getUnit().isInverseAngle()
+            >>> PhysicalField("1. inch").unit.isInverseAngle()
             0
         """
         return self.powers[7] == -1 and \
@@ -1774,13 +1845,13 @@ class PhysicalUnit:
         """
         Returns `True` if the unit is dimensionless or an angle
             
-            >>> PhysicalField("1. m/m").getUnit().isDimensionlessOrAngle()
+            >>> PhysicalField("1. m/m").unit.isDimensionlessOrAngle()
             1
-            >>> PhysicalField("1. deg").getUnit().isDimensionlessOrAngle()
+            >>> PhysicalField("1. deg").unit.isDimensionlessOrAngle()
             1
-            >>> PhysicalField("1. rad").getUnit().isDimensionlessOrAngle()
+            >>> PhysicalField("1. rad").unit.isDimensionlessOrAngle()
             1
-            >>> PhysicalField("1. inch").getUnit().isDimensionlessOrAngle()
+            >>> PhysicalField("1. inch").unit.isDimensionlessOrAngle()
             0
         """
         return self.isDimensionless() or self.isAngle()
@@ -1789,7 +1860,7 @@ class PhysicalUnit:
         """
         Set the name of the unit to `name`
         
-            >>> a = PhysicalField("1. m/s").getUnit()
+            >>> a = PhysicalField("1. m/s").unit
             >>> a
             <PhysicalUnit m/s>
             >>> a.setName('meterpersecond')
@@ -1803,10 +1874,10 @@ class PhysicalUnit:
         """
         Return the name of the unit
         
-            >>> PhysicalField("1. m").getUnit().name()
+            >>> PhysicalField("1. m").unit.name()
             'm'
             >>> (PhysicalField("1. m") / PhysicalField("1. s") 
-            ...  / PhysicalField("1. s")).getUnit().name()
+            ...  / PhysicalField("1. s")).unit.name()
             'm/s**2'
         """
         num = ''
@@ -1839,7 +1910,7 @@ def _findUnit(unit):
         <PhysicalUnit 1>
         >>> _findUnit(1.)
         <PhysicalUnit 1>
-        >>> _findUnit(PhysicalField("4 N*m").getUnit())
+        >>> _findUnit(PhysicalField("4 N*m").unit)
         <PhysicalUnit m*N>
         >>> _findUnit(2.)
         Traceback (most recent call last):
@@ -1849,7 +1920,7 @@ def _findUnit(unit):
 ##     print unit, type(unit)
     
     if type(unit) == type(''):
-        name = string.strip(unit)
+        name = unit.strip()
         if len(name) == 0 or unit == '1':
             unit = _unity
         else:
@@ -1882,17 +1953,17 @@ def _Scale(quantity, scaling):
     
     `quantity` can be a `PhysicalField`
     
-        >>> print round(_Scale(PhysicalField("1. inch"), PhysicalField("1. mm")), 6)
+        >>> print numerix.round_(_Scale(PhysicalField("1. inch"), PhysicalField("1. mm")), 6)
         25.4
         
     or a value-unit string convertable to a `PhysicalField`
     
-        >>> print round(_Scale("1. inch", PhysicalField("1. mm")), 6)
+        >>> print numerix.round_(_Scale("1. inch", PhysicalField("1. mm")), 6)
         25.4
         
     or a dimensionless number. A dimensionless number is left alone. 
            
-        >>> print round(_Scale(PhysicalField(2.), PhysicalField("1. mm")), 6)
+        >>> print numerix.round_(_Scale(PhysicalField(2.), PhysicalField("1. mm")), 6)
         2.0
         
     It is an error for the result to have dimensions.
@@ -1905,7 +1976,7 @@ def _Scale(quantity, scaling):
     
     quantity = PhysicalField(quantity)
 
-    if not quantity.getUnit().isDimensionless():
+    if not quantity.unit.isDimensionless():
         scaling = PhysicalField(scaling)
         # normalize quantity to scaling
         # error will be thrown if incompatible
@@ -1915,9 +1986,9 @@ def _Scale(quantity, scaling):
         # Automatically throws an error if it's not a number.
         dimensionless = quantity
                 
-    if isinstance(dimensionless,PhysicalField) and not dimensionless.getUnit().isDimensionless():
-        raise TypeError, `quantity.inBaseUnits().unit` + ' and ' \
-        + `scaling.inBaseUnits().unit` \
+    if isinstance(dimensionless,PhysicalField) and not dimensionless.unit.isDimensionless():
+        raise TypeError, repr(quantity.inBaseUnits().unit) + ' and ' \
+        + repr(scaling.inBaseUnits().unit) \
         + ' are incompatible'
         
     return dimensionless
@@ -1969,7 +2040,7 @@ for unit in _base_units:
     _unit_table[unit[0]] = unit[1]
 
 def _addUnit(name, unit):
-    if _unit_table.has_key(name):
+    if name in _unit_table:
         raise KeyError, 'Unit ' + name + ' already defined'
     if type(unit) == type(''):
         unit = eval(unit, _unit_table)
@@ -2129,9 +2200,9 @@ def _getUnitStrings():
     def _getSortedUnitStrings(unitDict):
         strings = []
         keys = unitDict.keys()
-        keys.sort(lambda x,y: cmp(x.lower(), y.lower()))
+        keys.sort(key=str.lower)
         for key in keys:
-            if unitDict.has_key(key):
+            if key in unitDict:
                 unit = unitDict[key]
                 if isinstance(unit, PhysicalUnit):
                     tmp = PhysicalField(value = 1, unit = unit)
@@ -2143,7 +2214,7 @@ def _getUnitStrings():
         
     def _deleteFactors(unit):
         for prefix, factor in _prefixes:
-            if working_table.has_key(prefix + unit.name()):
+            if (prefix + unit.name()) in working_table:
                 del working_table[prefix + unit.name()]
         
     
@@ -2152,7 +2223,7 @@ def _getUnitStrings():
     units.append("\nBase SI units::\n")
     units.append("\t%s" % ", ".join(_base_names))
     for name in _base_names:
-        if working_table.has_key(name):
+        if name in working_table:
             del working_table[name]
     for name, unit in _base_units:
         _deleteFactors(unit)
@@ -2164,7 +2235,7 @@ def _getUnitStrings():
     units.append("\nUnits derived from SI (accepting SI prefixes)::\n")
     derived = {}
     for key in working_table.keys():
-        if working_table.has_key(key):
+        if key in working_table:
             unit = working_table[key]
             if isinstance(unit, PhysicalUnit) and unit.factor == 1:
                 derived[unit.name()] = unit
@@ -2175,12 +2246,12 @@ def _getUnitStrings():
     units.append("\nOther units that accept SI prefixes::\n")
     prefixed = {}
     for key in working_table.keys():
-        if working_table.has_key(key):
+        if key in working_table:
             unit = working_table[key]
             isPrefixed = 1
             if isinstance(unit, PhysicalUnit):
                 for prefix, factor in _prefixes:
-                    if not working_table.has_key(prefix + key):
+                    if (prefix + key) not in working_table:
                         isPrefixed = 0
                         break
                 if isPrefixed:
@@ -2197,8 +2268,8 @@ def _getUnitStrings():
 __doc__ += _getUnitStrings()
 
 def _test(): 
-    import doctest
-    return doctest.testmod()
+    import fipy.tests.doctestPlus
+    return fipy.tests.doctestPlus.testmod()
     
 if __name__ == "__main__": 
 ##     print _getUnitStrings()
