@@ -30,22 +30,28 @@
  # #############################################################################
  ##
 
+__docformat__ = 'restructuredtext'
+
+__all__ = []
+
+import sys
+
 from fipy.variables.variable import Variable
-
 from fipy.tools import numerix
+from fipy.tools.decorators import getsetDeprecated
 
-def _OperatorVariableClass(baseClass=None):
+def _OperatorVariableClass(baseClass=object):
     class _OperatorVariable(baseClass):
         def __init__(self, op, var, opShape=(), canInline=True, unit=None, inlineComment=None, *args, **kwargs):
             self.op = op
             self.var = var
             self.opShape = opShape
-            self.unit = unit
+            self._unit = unit
             self.canInline = canInline  #allows for certain functions to opt out of --inline
             baseClass.__init__(self, value=None, *args, **kwargs)
             self.name = ''
             for var in self.var:    #C does not accept units
-                if not var.getUnit().isDimensionless():
+                if not var.unit.isDimensionless():
                     self.canInline = False
                     break
 
@@ -64,15 +70,15 @@ def _OperatorVariableClass(baseClass=None):
         
         def _calcValue(self):
             if not self.canInline:
-                return self._calcValuePy()
+                return self._calcValue_()
             else:
                 from fipy.tools import inline
-                return inline._optionalInline(self._calcValueIn, self._calcValuePy)
+                if inline.doInline:
+                    return self._execInline(comment=self.comment)
+                else:
+                    return self._calcValue_()
 
-        def _calcValueIn(self):
-            return self._execInline(comment=self.comment)
-
-        def _calcValuePy(self):
+        def _calcValue_(self):
             pass
 
         def _isCached(self):
@@ -105,7 +111,7 @@ def _OperatorVariableClass(baseClass=None):
                     result = repr(v)
                 elif style == "name":
                     if isinstance(v, Variable):
-                        result = v.getName()
+                        result = v.name
                         if len(result) == 0:
                             # The string form of a variable
                             # would probably be too long and messy.
@@ -124,9 +130,12 @@ def _OperatorVariableClass(baseClass=None):
                 elif style == "C":
                     if not v._isCached():
                         result = v._getCstring(argDict, id=id + str(i), freshen=freshen)
-                        v.value = None
+                        if isinstance(v, Variable):
+                            v._value = None
+                        else:
+                            v.value = None
                     else:
-                        result = v._getVariableClass()._getCstring(v, argDict,
+                        result = v._variableClass._getCstring(v, argDict,
                                                                    id=id + str(i),
                                                                    freshen=False)
                 else:
@@ -137,7 +146,10 @@ def _OperatorVariableClass(baseClass=None):
             if isinstance(self.op, numerix.ufunc):
                 return "%s(%s)" % (self.op.__name__, ", ".join([__var(i) for i in range(len(self.var))]))
             
-            bytecodes = [ord(byte) for byte in self.op.func_code.co_code]
+            if sys.version_info < (3,0):
+                bytecodes = [ord(byte) for byte in self.op.func_code.co_code]
+            else:
+                bytecodes = list(self.op.__code__.co_code)
                 
             def _popIndex():
                 return bytecodes.pop(0) + bytecodes.pop(0) * 256
@@ -188,23 +200,33 @@ def _OperatorVariableClass(baseClass=None):
                 elif opcode.opname[bytecode] == 'LOAD_DEREF':
                     free = self.op.func_code.co_cellvars + self.op.func_code.co_freevars
                     stack.append(free[_popIndex()])
-                elif unop.has_key(bytecode):
+                elif bytecode in unop:
                     stack.append(unop[bytecode] + '(' + stack.pop() + ')')
-                elif binop.has_key(bytecode):
+                elif bytecode in binop:
                     stack.append(stack.pop(-2) + " " + binop[bytecode] + " " + stack.pop())
                 else:
-                    raise SyntaxError, "Unknown bytecode: %s in %s: %s" % (`bytecode`, `[ord(byte) for byte in self.op.func_code.co_code]`,`"FIXME"`)
+                    raise SyntaxError, "Unknown bytecode: %s in %s: %s" % (
+                       repr(bytecode), 
+                       repr([ord(byte) for byte in self.op.func_code.co_code]),
+                       "FIXME")
                 
         def __repr__(self):
             return self._getRepresentation()
 
+        @getsetDeprecated
         def getName(self):
-            name = baseClass.getName(self)
+            return self.name
+
+        def _getName(self):
+            name = baseClass._getName(self)
             if len(name) == 0:
                 name = self._getRepresentation(style="name")
             return name
-            
-        def getShape(self):
+   
+        name = property(_getName, baseClass._setName)
+
+        @property
+        def shape(self):
             if self.opShape is not None:
                 return self.opShape
             else:
@@ -246,21 +268,21 @@ def _testBinOp(self):
         >>> (v1 / v2 - v3 * v4 + v1 * v4)._getRepresentation(style='C', id="")
         '(((var000[i] / var001[i]) - (var010[i] * var011[i])) + (var10[i] * var11[i]))'
         
-    Check that getUnit() works for a binOp
+    Check that unit works for a binOp
 
-        >>> (Variable(value="1 m") * Variable(value="1 s")).getUnit()
+        >>> (Variable(value="1 m") * Variable(value="1 s")).unit
         <PhysicalUnit s*m>
 
-        >>> (Variable(value="1 m") / Variable(value="0 s")).getUnit()
+        >>> (Variable(value="1 m") / Variable(value="0 s")).unit
         <PhysicalUnit m/s>
 
-        >>> a = -((Variable() * Variable()).sin())
+        >>> a = -(numerix.sin(Variable() * Variable()))
 
     Check that getTypeCode() works as expected.
 
         >>> a = Variable(1.) * Variable(1)
-        >>> a.getsctype()
-        <type 'numpy.float64'>
+        >>> print a.getsctype() == numerix.float64
+        1
 
     The following test is to correct an `--inline` bug that was
     being thrown by the Cahn-Hilliard example. The fix for this
@@ -274,31 +296,31 @@ def _testBinOp(self):
         >>> ##        self.var[counter].value=None
 
     This is the test that fails if the last line above is removed
-    from `_getRepresentation()`, the `binOp.getValue()` statement
+    from `_getRepresentation()`, the `binOp.value` statement
     below will return `1.0` and not `0.5`.
         
         >>> from fipy import numerix
         >>> def doBCs(binOp):
         ...     unOp1 = -binOp
-        ...     print binOp.getValue()
+        ...     print binOp.value
         >>> var = Variable(1.)
         >>> binOp = 1. * var
         >>> unOp = -binOp
-        >>> print unOp.getValue()
+        >>> print unOp.value
         -1.0
         >>> doBCs(binOp)
         1.0
-        >>> var.setValue(0.5)
-        >>> print unOp.getValue()
+        >>> var.value = (0.5)
+        >>> print unOp.value
         -0.5
         >>> unOp2 = -binOp
-        >>> print binOp.getValue()
+        >>> print binOp.value
         0.5
 
         >>> from fipy.variables.cellVariable import CellVariable
         >>> from fipy.variables.faceVariable import FaceVariable
         
-        >>> from fipy.meshes.grid2D import Grid2D
+        >>> from fipy.meshes import Grid2D
         >>> mesh = Grid2D(nx=3)
         
         
@@ -332,7 +354,7 @@ def _testBinOp(self):
          [0 2 6]]
         >>> print isinstance(vcvXcv, CellVariable)
         1
-        >>> print vcvXcv.getRank()
+        >>> print vcvXcv.rank
         1
         >>> cvXvcv = cv * vcv
         >>> print cvXvcv
@@ -340,7 +362,7 @@ def _testBinOp(self):
          [0 2 6]]
         >>> print isinstance(cvXvcv, CellVariable)
         1
-        >>> print cvXvcv.getRank()
+        >>> print cvXvcv.rank
         1
 
     `CellVariable` * rank-1 `FaceVariable`
@@ -381,7 +403,7 @@ def _testBinOp(self):
          [0 2 4]]
         >>> print isinstance(cvXv2, CellVariable)
         1
-        >>> print cvXv2.getRank()
+        >>> print cvXv2.rank
         1
         >>> v2Xcv = (3,2) * cv
         >>> print v2Xcv
@@ -393,7 +415,7 @@ def _testBinOp(self):
          [0 2 4]]
         >>> print isinstance(v2Xcv, CellVariable)
         1
-        >>> print v2Xcv.getRank()
+        >>> print v2Xcv.rank
         1
         
         >>> cvXv3 = cv * (3,2,1)
@@ -455,7 +477,7 @@ def _testBinOp(self):
          [0 2 4]]
         >>> print isinstance(cvXv2v, CellVariable)
         1
-        >>> print cvXv2v.getRank()
+        >>> print cvXv2v.rank
         1
         >>> v2vXcv = Variable(value=(3,2)) * cv
         >>> v2vXcv = Variable(value=((3,),(2,))) * cv
@@ -464,7 +486,7 @@ def _testBinOp(self):
          [0 2 4]]
         >>> print isinstance(v2vXcv, CellVariable)
         1
-        >>> print v2vXcv.getRank()
+        >>> print v2vXcv.rank
         1
         
         >>> cvXv3v = cv * Variable(value=(3,2,1))
@@ -490,13 +512,13 @@ def _testBinOp(self):
 
     `CellVariable` * CellGradVariable
     
-        >>> cvXcgv = cv * cv.getGrad()
+        >>> cvXcgv = cv * cv.grad
         >>> print cvXcgv
         [[ 0.  1.  1.]
          [ 0.  0.  0.]]
         >>> print isinstance(cvXcgv, CellVariable)
         1
-        >>> print cvXcgv.getRank()
+        >>> print cvXcgv.rank
         1
         
     `FaceVariable` * FaceVariable
@@ -526,7 +548,7 @@ def _testBinOp(self):
          [ 0  2  6 12 12 20 30 63 48 27]]
         >>> print isinstance(vfvXfv, FaceVariable)
         1
-        >>> print vfvXfv.getRank()
+        >>> print vfvXfv.rank
         1
         >>> fvXvfv = fv * vfv
         >>> print fvXvfv
@@ -534,7 +556,7 @@ def _testBinOp(self):
          [ 0  2  6 12 12 20 30 63 48 27]]
         >>> print isinstance(fvXvfv, FaceVariable)
         1
-        >>> print fvXvfv.getRank()
+        >>> print fvXvfv.rank
         1
 
     `FaceVariable` * Scalar
@@ -562,7 +584,7 @@ def _testBinOp(self):
          [ 0  2  4  6  8 10 12 14 16 18]]
         >>> print isinstance(fvXv2, FaceVariable)
         1
-        >>> print fvXv2.getRank()
+        >>> print fvXv2.rank
         1
         >>> v2Xfv = (3,2) * fv
         >>> print v2Xfv
@@ -574,7 +596,7 @@ def _testBinOp(self):
          [ 0  2  4  6  8 10 12 14 16 18]]
         >>> print isinstance(v2Xfv, FaceVariable)
         1
-        >>> print v2Xfv.getRank()
+        >>> print v2Xfv.rank
         1
         
         >>> fvXv3 = fv * (3,2,1) #doctest: +IGNORE_EXCEPTION_DETAIL
@@ -622,7 +644,7 @@ def _testBinOp(self):
          [ 0  2  4  6  8 10 12 14 16 18]]
         >>> print isinstance(fvXv2v, FaceVariable)
         1
-        >>> print fvXv2v.getRank()
+        >>> print fvXv2v.rank
         1
         >>> v2vXfv = Variable(value=(3,2)) * fv
         >>> print v2vXfv
@@ -634,7 +656,7 @@ def _testBinOp(self):
          [ 0  2  4  6  8 10 12 14 16 18]]
         >>> print isinstance(v2vXfv, FaceVariable)
         1
-        >>> print v2vXfv.getRank()
+        >>> print v2vXfv.rank
         1
         
         >>> fvXv3v = fv * Variable(value=(3,2,1)) #doctest: +IGNORE_EXCEPTION_DETAIL
@@ -667,7 +689,7 @@ def _testBinOp(self):
          [1 4 9]]
         >>> print isinstance(vcvXvcv, CellVariable)
         1
-        >>> print vcvXvcv.getRank()
+        >>> print vcvXvcv.rank
         1
 
     rank-1 `CellVariable` * rank-1 `FaceVariable`
@@ -689,7 +711,7 @@ def _testBinOp(self):
          [3 6 9]]
         >>> print isinstance(vcvXs, CellVariable)
         1
-        >>> print vcvXs.getRank()
+        >>> print vcvXs.rank
         1
         >>> sXvcv = 3 * vcv
         >>> print sXvcv
@@ -697,7 +719,7 @@ def _testBinOp(self):
          [3 6 9]]
         >>> print isinstance(vcvXs, CellVariable)
         1
-        >>> print vcvXs.getRank()
+        >>> print vcvXs.rank
         1
 
     rank-1 `CellVariable` * Vector
@@ -712,7 +734,7 @@ def _testBinOp(self):
          [2 4 6]]
         >>> print isinstance(vcvXv2, CellVariable)
         1
-        >>> print vcvXv2.getRank()
+        >>> print vcvXv2.rank
         1
         >>> v2Xvcv = (3,2) * vcv
         >>> print v2Xvcv
@@ -724,7 +746,7 @@ def _testBinOp(self):
          [2 4 6]]
         >>> print isinstance(v2Xvcv, CellVariable)
         1
-        >>> print v2Xvcv.getRank()
+        >>> print v2Xvcv.rank
         1
         
         >>> vcvXv3 = vcv * (3,2,1)
@@ -733,7 +755,7 @@ def _testBinOp(self):
          [3 4 3]]
         >>> isinstance(vcvXv3, CellVariable)
         1
-        >>> print vcvXv3.getRank()
+        >>> print vcvXv3.rank
         1
         >>> v3Xvcv = (3,2,1) * vcv 
         >>> print v3Xvcv
@@ -741,7 +763,7 @@ def _testBinOp(self):
          [3 4 3]]
         >>> isinstance(v3Xvcv, CellVariable)
         1
-        >>> print v3Xvcv.getRank()
+        >>> print v3Xvcv.rank
         1
 
         >>> vcvXv4 = vcv * (3,2,1,0) #doctest: +IGNORE_EXCEPTION_DETAIL
@@ -761,7 +783,7 @@ def _testBinOp(self):
          [3 6 9]]
         >>> print isinstance(vcvXsv, CellVariable)
         1
-        >>> print vcvXsv.getRank()
+        >>> print vcvXsv.rank
         1
         >>> svXvcv = Variable(value=3) * vcv
         >>> print svXvcv
@@ -769,7 +791,7 @@ def _testBinOp(self):
          [3 6 9]]
         >>> print isinstance(svXvcv, CellVariable)
         1
-        >>> print svXvcv.getRank()
+        >>> print svXvcv.rank
         1
 
     rank-1 `CellVariable` * `Variable` Vector
@@ -784,7 +806,7 @@ def _testBinOp(self):
          [2 4 6]]
         >>> print isinstance(vcvXv2v, CellVariable)
         1
-        >>> print vcvXv2v.getRank()
+        >>> print vcvXv2v.rank
         1
         >>> v2vXvcv = Variable(value=(3,2)) * vcv
         >>> print v2vXvcv
@@ -796,7 +818,7 @@ def _testBinOp(self):
          [2 4 6]]
         >>> print isinstance(v2vXvcv, CellVariable)
         1
-        >>> print v2vXvcv.getRank()
+        >>> print v2vXvcv.rank
         1
         
         >>> vcvXv3v = vcv * Variable(value=(3,2,1))
@@ -805,7 +827,7 @@ def _testBinOp(self):
          [3 4 3]]
         >>> isinstance(vcvXv3v, CellVariable)
         1
-        >>> print vcvXv3v.getRank()
+        >>> print vcvXv3v.rank
         1
         >>> v3vXvcv = Variable(value=(3,2,1)) * vcv 
         >>> print v3vXvcv
@@ -813,7 +835,7 @@ def _testBinOp(self):
          [3 4 3]]
         >>> isinstance(v3vXvcv, CellVariable)
         1
-        >>> print v3vXvcv.getRank()
+        >>> print v3vXvcv.rank
         1
 
         >>> vcvXv4v = vcv * Variable(value=(3,2,1,0)) #doctest: +IGNORE_EXCEPTION_DETAIL
@@ -834,7 +856,7 @@ def _testBinOp(self):
          [ 1  4  9 16  9 16 25 81 36  9]]
         >>> isinstance(vfvXvfv, FaceVariable)
         1
-        >>> print vfvXvfv.getRank()
+        >>> print vfvXvfv.rank
         1
 
     rank-1 `FaceVariable` * Scalar
@@ -845,7 +867,7 @@ def _testBinOp(self):
          [ 3  6  9 12  9 12 15 27 18  9]]
         >>> print isinstance(vfvXs, FaceVariable)
         1
-        >>> print vfvXs.getRank()
+        >>> print vfvXs.rank
         1
         >>> sXvfv = 3 * vfv
         >>> print sXvfv
@@ -853,7 +875,7 @@ def _testBinOp(self):
          [ 3  6  9 12  9 12 15 27 18  9]]
         >>> print isinstance(sXvfv, FaceVariable)
         1
-        >>> print sXvfv.getRank()
+        >>> print sXvfv.rank
         1
 
     rank-1 `FaceVariable` * Vector
@@ -864,7 +886,7 @@ def _testBinOp(self):
          [ 2  4  6  8  6  8 10 18 12  6]]
         >>> print isinstance(vfvXv2, FaceVariable)
         1
-        >>> print vfvXv2.getRank()
+        >>> print vfvXv2.rank
         1
         >>> v2Xvfv = (3,2) * vfv
         >>> print v2Xvfv
@@ -872,7 +894,7 @@ def _testBinOp(self):
          [ 2  4  6  8  6  8 10 18 12  6]]
         >>> print isinstance(v2Xvfv, FaceVariable)
         1
-        >>> print v2Xvfv.getRank()
+        >>> print v2Xvfv.rank
         1
         
         >>> vfvXv3 = vfv * (2,1,0) #doctest: +IGNORE_EXCEPTION_DETAIL
@@ -891,7 +913,7 @@ def _testBinOp(self):
          [ 9 16 21 24 15 16 15 18  6  0]]
         >>> isinstance(vfvXv10, FaceVariable)
         1
-        >>> print vfvXv10.getRank()
+        >>> print vfvXv10.rank
         1
         >>> v10Xvfv = (9,8,7,6,5,4,3,2,1,0) * vfv
         >>> print v10Xvfv
@@ -899,7 +921,7 @@ def _testBinOp(self):
          [ 9 16 21 24 15 16 15 18  6  0]]
         >>> isinstance(v10Xvfv, FaceVariable)
         1
-        >>> print v10Xvfv.getRank()
+        >>> print v10Xvfv.rank
         1
 
     rank-1 `FaceVariable` * `Variable` Scalar
@@ -910,7 +932,7 @@ def _testBinOp(self):
          [ 3  6  9 12  9 12 15 27 18  9]]
         >>> print isinstance(vfvXsv, FaceVariable)
         1
-        >>> print vfvXsv.getRank()
+        >>> print vfvXsv.rank
         1
         >>> svXvfv = Variable(value=3) * vfv
         >>> print svXvfv
@@ -918,7 +940,7 @@ def _testBinOp(self):
          [ 3  6  9 12  9 12 15 27 18  9]]
         >>> print isinstance(svXvfv, FaceVariable)
         1
-        >>> print svXvfv.getRank()
+        >>> print svXvfv.rank
         1
 
     rank-1 `FaceVariable` * `Variable` Vector
@@ -929,7 +951,7 @@ def _testBinOp(self):
          [ 2  4  6  8  6  8 10 18 12  6]]
         >>> print isinstance(vfvXv2v, FaceVariable)
         1
-        >>> print vfvXv2v.getRank()
+        >>> print vfvXv2v.rank
         1
         >>> v2vXvfv = Variable(value=(3,2)) * vfv
         >>> print v2vXvfv
@@ -937,7 +959,7 @@ def _testBinOp(self):
          [ 2  4  6  8  6  8 10 18 12  6]]
         >>> print isinstance(v2vXvfv, FaceVariable)
         1
-        >>> print v2vXvfv.getRank()
+        >>> print v2vXvfv.rank
         1
         
         >>> vfvXv3v = vfv * Variable(value=(2,1,0)) #doctest: +IGNORE_EXCEPTION_DETAIL
@@ -956,7 +978,7 @@ def _testBinOp(self):
          [ 9 16 21 24 15 16 15 18  6  0]]
         >>> isinstance(vfvXv10v, FaceVariable)
         1
-        >>> print vfvXv10v.getRank()
+        >>> print vfvXv10v.rank
         1
         >>> v10vXvfv = Variable(value=(9,8,7,6,5,4,3,2,1,0)) * vfv
         >>> print v10vXvfv
@@ -964,7 +986,7 @@ def _testBinOp(self):
          [ 9 16 21 24 15 16 15 18  6  0]]
         >>> isinstance(v10vXvfv, FaceVariable)
         1
-        >>> print v10vXvfv.getRank()
+        >>> print v10vXvfv.rank
         1
         
         
@@ -1075,17 +1097,17 @@ def _testBinOp(self):
         ...         Variable.__init__(self)
         ...         self.var = self._requires(var)
         ...     def _calcValue(self):
-        ...         return self.var.getValue()
+        ...         return self.var.value
 
         >>> coeff = Variable()
         >>> alpha = Alpha(-coeff / 1)
-        >>> print numerix.allclose(alpha.getValue(), 0.0)
+        >>> print numerix.allclose(alpha.value, 0.0)
         True
-        >>> coeff.setValue(-10.0)
-        >>> print numerix.allclose(alpha.getValue(), 10)
+        >>> coeff.value = (-10.0)
+        >>> print numerix.allclose(alpha.value, 10)
         True
-        >>> coeff.setValue(10.0)
-        >>> print numerix.allclose(alpha.getValue(), -10)
+        >>> coeff.value = (10.0)
+        >>> print numerix.allclose(alpha.value, -10)
         True
 
     Test to prevent divide by zero evaluation before value is
@@ -1126,7 +1148,7 @@ def _testBinOp(self):
         >>> print a
         [[ 0  1  2  3  4  5  6  7  8]
          [ 0 -1 -2 -3 -4 -5 -6 -7 -8]]
-        >>> x, y = mesh.getCellCenters()
+        >>> x, y = mesh.cellCenters
         >>> v1[(x == 0.5) & (y == 0.5)] = 0
         >>> print a
         [[ 0  1  2  3  4  5  6  7  8]
@@ -1136,8 +1158,8 @@ def _testBinOp(self):
     pass
 
 def _test(): 
-    import doctest
-    return doctest.testmod()
+    import fipy.tests.doctestPlus
+    return fipy.tests.doctestPlus.testmod()
     
 if __name__ == "__main__": 
     _test()
